@@ -17,12 +17,16 @@
 
 """Representation of packages in the application stack."""
 
+import re
 import logging
 import typing
 from copy import copy
 
 import attr
 import semantic_version as semver
+from packaging.version import parse as parse_version
+from packaging.utils import canonicalize_name
+
 
 from .exceptions import UnsupportedConfiguration
 from .exceptions import PipfileParseError
@@ -32,12 +36,27 @@ from .source import Source
 _LOGGER = logging.getLogger(__name__)
 
 
+_RE_NORMALIZE_PYTHON_PACKAGE_NAME = re.compile(r"[-_.]+")
+
+
+def _normalize_python_package_name(package_name: str) -> str:
+    """Normalize Python package name based on PEP-0503."""
+    # Make sure we have normalized names in the graph database according to PEP:
+    #   https://www.python.org/dev/peps/pep-0503/#normalized-names
+    return canonicalize_name(package_name)
+
+
+def _normalize_python_package_version(package_version: str) -> str:
+    """Normalize Python package version based on PEP-440."""
+    return str(parse_version(package_version))
+
+
 @attr.s(slots=True)
 class PackageVersion:
     """A package version as described in the Pipfile.lock entry."""
 
-    name = attr.ib(type=str)
-    version = attr.ib(type=str)
+    name = attr.ib(type=str, converter=_normalize_python_package_name)
+    version = attr.ib(type=str, converter=_normalize_python_package_version)
     develop = attr.ib(type=bool)
     index = attr.ib(default=None, type=Source)
     hashes = attr.ib(default=attr.Factory(list))
@@ -74,6 +93,22 @@ class PackageVersion:
 
         return self.semantic_version > other.semantic_version
 
+    @classmethod
+    def normalize_python_package_name(cls, package_name: str) -> str:
+        """Normalize Python package name based on PEP-0503.
+
+        https://www.python.org/dev/peps/pep-0503/#normalized-names
+        """
+        return _normalize_python_package_name(package_name)
+
+    @classmethod
+    def normalize_python_package_version(self, package_version: str) -> str:
+        """Normalize Python package version based on PEP-440.
+
+        https://www.python.org/dev/peps/pep-0440/#normalization
+        """
+        return _normalize_python_package_version(package_version)
+
     @classmethod
     def from_model(cls, model, *, develop: bool = False):
         """Convert database model representation to object representation."""
@@ -126,20 +161,33 @@ class PackageVersion:
                     f"Cannot get semantic version for not-locked package {self.name} in version {self.version}"
                 )
 
-        self._semantic_version = self.parse_semantic_version(self.locked_version)
+        try:
+            self._semantic_version = self.parse_semantic_version(self.locked_version, _package_name=self.name)
+        except ValueError:
+            # A simple workaround for leading zeros when parsing semver - e.g. 3.01.2.
+            parts = self.locked_version.split('.')
+            version = ".".join(map(str, map(int, parts)))
+            self._semantic_version = self.parse_semantic_version(version, self.name)
+
         return self._semantic_version
 
     @staticmethod
-    def parse_semantic_version(version_identifier: str) -> semver.Version:
+    def parse_semantic_version(version_identifier: str, _package_name: str = None) -> semver.Version:
         """Parse the given version identifier into a semver representation."""
         try:
             semantic_version = semver.Version(version_identifier)
         except Exception as exc:
             semantic_version = semver.Version.coerce(version_identifier)
-            _LOGGER.debug(
-                f"Cannot determine semantic version {version_identifier}, "
-                f"approximated version is {semantic_version}: {str(exc)}"
-            )
+            if _package_name:
+                _LOGGER.debug(
+                    f"Cannot determine semantic version {version_identifier}, "
+                    f"approximated version is {semantic_version}: {str(exc)}"
+                )
+            else:
+                _LOGGER.debug(
+                    f"Cannot determine semantic version {version_identifier} of package {_package_name}, "
+                    f"approximated version is {semantic_version}: {str(exc)}"
+                )
 
         return semantic_version
 
